unit ShadeLabel;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, StdCtrls,
     Gaus, Graphics;

type
  TShadeLabel = class(TGraphicControl)
  private
    { Private declarations }
    FExtent : Double;
    FPaintShade : Boolean;
    FColor : TColor;
    FOnMouseEnter : TNotifyEvent;
    FOnMouseLeave : TNotifyEvent;
    procedure SetExtent(Value: Double);
    procedure SetColor(Value: TColor);
    procedure SetPaintShade(Value: Boolean);
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
  protected
    { Protected declarations }
    procedure Paint; override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    property Canvas;
  published
    { Published declarations }
    property Caption;
    property Font;
    property Color: TColor read FColor write SetColor;
    property PaintShade: Boolean read FPaintShade write SetPaintShade;
    property Extent: Double read FExtent write SetExtent;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
    property OnMouseLeave: TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
  end;

procedure Register;

implementation

constructor TShadeLabel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  SetPaintShade(True);
  SetExtent(1);
  SetColor(clRed);
  Font.Name := 'Times New Roman';
  Font.Size := 20;
end;

procedure TShadeLabel.SetExtent(Value: double);
begin
If Value <> Extent then
  FExtent := Value;
RePaint;
end;

procedure TShadeLabel.SetPaintShade(Value: Boolean);
begin
If Value <> PaintShade then
  FPaintShade := Value;
RePaint;
end;

procedure TShadeLabel.SetColor(Value: TColor);
begin
If Value <> Color then
  FColor := Value;
RePaint;
end;

procedure TShadeLabel.CMTextChanged(var Message: TMessage);
begin
  Invalidate;
  RePaint;
end;

procedure TShadeLabel.CMFontChanged(var Message: TMessage);
begin
  Invalidate;
  RePaint;
end;


procedure TShadeLabel.CMMouseEnter(var Message: TMessage);
begin
  inherited;
  if Assigned(FOnMouseEnter) then
    FOnMouseEnter(Self);
end;

procedure TShadeLabel.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  if Assigned(FOnMouseEnter) then
    FOnMouseLeave(Self);
end;

procedure TShadeLabel.Paint;
var   Rect : TRect;
 DrawStyle : LongInt;
 BMP       : TBitmap;
begin
Canvas.Font := Font;
BMP := TBitmap.Create;
BMP.Width  := Canvas.TextWidth(Caption);
BMP.Height := Canvas.TextHeight(Caption);
Width := BMP.Width; Height := BMP.Height;
Rect := ClientRect;
DrawStyle := DT_EXPANDTABS;
BMP.Canvas.Font := Font;
BMP.PixelFormat := pf24Bit;
Canvas.Brush.Style := bsClear;
BMP.Canvas.Brush.Color := Color;
BMP.Canvas.Rectangle(-1,-1,Width+1,Height+1);
If Color = clFuchsia then DrawText(Canvas.Handle,PChar(Caption),Length(Text),Rect,DrawStyle);
If Color = clFuchsia then BMP.Canvas.CopyRect(Rect,Canvas,Rect) else
   DrawText(BMP.Canvas.Handle,PChar(Caption),Length(Text),Rect,DrawStyle);
GBlur(BMP, Extent);
If PaintShade then Canvas.Draw(0,0,BMP);
BMP.Free;
end;

procedure Register;
begin
  RegisterComponents('Samples', [TShadeLabel]);
end;

end.
